@@ -5,15 +5,19 @@ import logging
 import os
 import threading
 import traceback
-from typing import Union, Dict, List
+from typing import Union, Dict, List, Optional, TypeVar, Callable, Any, cast
 
-from bottle import Bottle, redirect, request, response, static_file
+from bottle import Bottle, BaseResponse, redirect, request, response, static_file
+from optuna.exceptions import DuplicatedStudyError
 from optuna.storages import BaseStorage, get_storage
 from optuna.trial import FrozenTrial
 from optuna.study import StudyDirection, StudySummary
 
 from . import serializer
 
+BottleViewReturn = Union[str, bytes, Dict[str, Any], BaseResponse]
+BottleView = TypeVar("BottleView", bound=Callable[..., BottleViewReturn])
+
 logger = logging.getLogger(__name__)
 
 BASE_DIR = os.path.dirname(os.path.abspath(__file__))
@@ -46,9 +50,9 @@ trials_cache: Dict[int, List[FrozenTrial]] = {}
 trials_last_fetched_at: Dict[int, datetime] = {}
 
 
-def handle_json_api_exception(view):
+def handle_json_api_exception(view: BottleView) -> BottleView:
     @functools.wraps(view)
-    def decorated(*args, **kwargs):
+    def decorated(*args: List[Any], **kwargs: Dict[str, Any]) -> BottleViewReturn:
         try:
             response_body = view(*args, **kwargs)
             return response_body
@@ -59,15 +63,16 @@ def handle_json_api_exception(view):
             logger.error(f"Exception: {e}\n{stacktrace}")
             return json.dumps({"reason": "internal server error"})
 
-    return decorated
+    return cast(BottleView, decorated)
 
 
-def get_study_summary(storage: BaseStorage, study_id: int) -> StudySummary:
+def get_study_summary(storage: BaseStorage, study_id: int) -> Optional[StudySummary]:
     summaries = storage.get_all_study_summaries()
     for summary in summaries:
         if summary._study_id != study_id:
             continue
         return summary
+    return None
 
 
 def get_trials(
@@ -93,72 +98,96 @@ def create_app(storage_or_url: Union[str, BaseStorage]) -> Bottle:
     app = Bottle()
     storage = get_storage(storage_or_url)
 
+    @app.hook("before_request")
+    def remove_trailing_slashes_hook() -> None:
+        request.environ["PATH_INFO"] = request.environ["PATH_INFO"].rstrip("/")
+
     @app.get("/")
-    def index():
+    def index() -> BottleViewReturn:
         return redirect("/dashboard", 302)  # Status Found
 
     # Accept any following paths for client-side routing
     @app.get("/dashboard<:re:(/.*)?>")
-    def dashboard():
+    def dashboard() -> BottleViewReturn:
         response.content_type = "text/html"
         return INDEX_HTML
 
     @app.get("/api/studies")
     @handle_json_api_exception
-    def list_study_summaries():
+    def list_study_summaries() -> BottleViewReturn:
         response.content_type = "application/json"
         summaries = [
             serializer.serialize_study_summary(summary)
             for summary in storage.get_all_study_summaries()
         ]
-        return json.dumps(
-            {
-                "study_summaries": summaries,
-            }
-        )
+        return {
+            "study_summaries": summaries,
+        }
 
     @app.post("/api/studies")
     @handle_json_api_exception
-    def create_study():
+    def create_study() -> BottleViewReturn:
         response.content_type = "application/json"
 
         study_name = request.json.get("study_name", None)
-        direction = request.json.get("direction", None)
-        if study_name is None or direction not in ("minimize", "maximize"):
+        directions = request.json.get("directions", [])
+        if (
+            study_name is None
+            or len(directions) == 0
+            or not all([d in ("minimize", "maximize") for d in directions])
+        ):
             response.status = 400  # Bad request
             return {"reason": "You need to set study_name and direction"}
 
-        study_id = storage.create_new_study(study_name)
-        if direction.lower() == "maximize":
-            storage.set_study_direction(study_id, StudyDirection.MAXIMIZE)
-        else:
-            storage.set_study_direction(study_id, StudyDirection.MINIMIZE)
+        try:
+            study_id = storage.create_new_study(study_name)
+        except DuplicatedStudyError:
+            response.status = 400  # Bad request
+            return {"reason": f"'{study_name}' is already exists"}
+
+        storage.set_study_directions(
+            study_id,
+            [
+                StudyDirection.MAXIMIZE
+                if d.lower() == "maximize"
+                else StudyDirection.MINIMIZE
+                for d in directions
+            ],
+        )
 
         summary = get_study_summary(storage, study_id)
+        if summary is None:
+            response.status = 500  # Internal server error
+            return {"reason": "Failed to create study"}
         response.status = 201  # Created
-        return json.dumps(
-            {"study_summary": serializer.serialize_study_summary(summary)}
-        )
+        return {"study_summary": serializer.serialize_study_summary(summary)}
 
     @app.delete("/api/studies/<study_id:int>")
     @handle_json_api_exception
-    def delete_study(study_id: int):
+    def delete_study(study_id: int) -> BottleViewReturn:
         response.content_type = "application/json"
 
-        storage.delete_study(study_id)
+        try:
+            storage.delete_study(study_id)
+        except KeyError:
+            response.status = 404  # Not found
+            return {"reason": f"study_id={study_id} is not found"}
         response.status = 204  # No content
         return ""
 
     @app.get("/api/studies/<study_id:int>")
     @handle_json_api_exception
-    def get_study_detail(study_id: int):
+    def get_study_detail(study_id: int) -> BottleViewReturn:
         response.content_type = "application/json"
         summary = get_study_summary(storage, study_id)
+        if summary is None:
+            response.status = 404  # Not found
+            return {"reason": f"study_id={study_id} is not found"}
         trials = get_trials(storage, study_id)
-        return json.dumps(serializer.serialize_study_detail(summary, trials))
+        return serializer.serialize_study_detail(summary, trials)
 
     @app.get("/static/<filename:path>")
-    def send_static(filename):
+    def send_static(filename: str) -> BottleViewReturn:
         return static_file(filename, root=STATIC_DIR)
 
     return app
